/* pvfs_mount.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

 */

static char pvfs_mount_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_mount.c,v 1.4 2000/01/13 05:56:06 rbross Exp $";

/* This file contains PVFS library call for mount.   		*/
#include <lib.h>
#include <meta.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_mount(const char* specialfile, char* dir, const char*
	filesystemtype, unsigned long rwflag, const void* data)
{
	int s, i, len; 
	mreq req;
	mack ack;
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;

	if (pvfs_checks_disabled) return(syscall(SYS_mount, specialfile,
		dir, filesystemtype, rwflag, data));

	/* Check to see if .pvfsdir exists in directory  	*/
	if ((i = pvfs_detect(dir, &fn, &saddr, &fs_ino)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return (syscall(SYS_mount, specialfile, dir, filesystemtype,
				rwflag, data));
	}

	/* Prepare request for file system  */
	req.dsize = strlen(fn);
	req.type = MGR_MOUNT;
	req.req.mount.fsname = 0;

	/* send request to manager */
	if ((s = send_mreq_saddr(saddr, &req, fn, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}

	return (0);
}
