/* pvfs-test.c
 *
 * This is derived from code given to me by Rajeev Thakur.  Dunno where
 * it originated.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <pvfs.h>
#include <sys/time.h>
#ifdef __USE_MPI__
#include <mpi.h>
#endif

#define AMODE O_RDWR | O_CREAT | O_META

/* DEFAULT VALUES FOR OPTIONS */
int64_t opt_block     = 1048576*16;
int     opt_iter      = 1;
int     opt_stripe    = -1;
int     opt_correct   = 0;
char    opt_file[256] = "/pvfs/test.out\0";

int parse_args(int argc, char **argv);
double Wtime(void);

extern fdesc_p pfds[];
extern int errno;
extern int debug_on;

extern char *optarg;
extern int optind, opterr;

main(int argc, char **argv)
{
	char *buf, *tmp, *buf2, *tmp2, *check;
	int fd, i, j, mynod=0, nprocs=1, err, my_correct = 1, correct, myerrno;
	double stim, etim, write_tim, new_write_tim;
	double min_write_tim=10000000.0, write_bw;
	double read_tim, new_read_tim;
	double min_read_tim=10000000.0, read_bw;
	struct pvfs_stat pstat = {0,-1,16384,0,0};

#ifdef __USE_MPI__
	MPI_Init(&argc,&argv);
	MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
	MPI_Comm_rank(MPI_COMM_WORLD, &mynod);
#endif

	parse_args(argc, argv);
	pstat.ssize = opt_stripe;

	if (!(tmp = (char *) malloc(opt_block + 256))) {
		perror("malloc");
		goto die_jar_jar_die;
	}
	buf = tmp + 128 - (((long)tmp) % 128);  /* align buffer */

	if (opt_correct) {
		if (!(tmp2 = (char *) malloc(opt_block + 256))) {
			perror("malloc2");
			goto die_jar_jar_die;
		}
		buf2 = tmp + 128 - (((long)tmp) % 128);
	}

	for (j=0; j < opt_iter; j++) {
		if ((fd = pvfs_open(opt_file, AMODE, 0644, &pstat)) < 0) {
			fprintf(stderr, "node %d, open error: %s\n", mynod, strerror(errno));
			goto die_jar_jar_die;
		}

	   pvfs_lseek(fd, mynod*opt_block, SEEK_SET);

		if (opt_correct) /* fill in buffer for iteration */ {
			for (i=mynod+j, check=buf; i<opt_block; i++,check++) *check=(char)i;
		}

#ifdef __USE_MPI__
	   MPI_Barrier(MPI_COMM_WORLD);
	   stim = MPI_Wtime();
#else
		stim = Wtime();
#endif
	   err = pvfs_write(fd, buf, opt_block);
		myerrno = errno;
#ifdef __USE_MPI__
	   etim = MPI_Wtime();
#else
		etim = Wtime();
#endif
	   write_tim = etim - stim;

	   if (err < 0) fprintf(stderr, "node %d, write error, loc = %Ld: %s\n",
			mynod, mynod*opt_block, strerror(myerrno));

	   if (pvfs_close(fd) < 0) {
			fprintf(stderr, "node %d, close error after write\n", mynod);
		}
	    
#ifdef __USE_MPI__
	   MPI_Barrier(MPI_COMM_WORLD);
#endif
	   if ((fd = pvfs_open(opt_file, AMODE, 0644, &pstat)) < 0) {
			fprintf(stderr, "node %d, open error: %s\n", mynod, strerror(errno));
			goto die_jar_jar_die;
		}

	   pvfs_lseek(fd, mynod*opt_block, SEEK_SET);

#ifdef __USE_MPI__
	   MPI_Barrier(MPI_COMM_WORLD);
	   stim = MPI_Wtime();
#else
		stim = Wtime();
#endif
		if (!opt_correct)
			err = pvfs_read(fd, buf, opt_block);
		else
			err = pvfs_read(fd, buf2, opt_block); /* read into second buffer */
		myerrno = errno;
#ifdef __USE_MPI
	   etim = MPI_Wtime();
#else
		etim = Wtime();
#endif
	   read_tim = etim - stim;

	   if (err < 0) fprintf(stderr, "node %d, read error, loc = %Ld: %s\n",
			mynod, mynod*opt_block, strerror(myerrno));

		if (opt_correct && memcmp(buf, buf2, opt_block)) {
			fprintf(stderr, "node %d, correctness test failed\n", mynod);
			my_correct = 0;
#ifdef __USE_MPI__
			MPI_Allreduce(&my_correct, &correct, 1, MPI_INT, MPI_MIN,
				MPI_COMM_WORLD);
#endif
		}

	   pvfs_close(fd);
	    
#ifdef __USE_MPI__
	   MPI_Allreduce(&read_tim, &new_read_tim, 1, MPI_DOUBLE, MPI_MAX,
			MPI_COMM_WORLD);
#else
		new_read_tim = read_tim;
#endif
	   min_read_tim = (new_read_tim < min_read_tim) ? 
			new_read_tim : min_read_tim;

#ifdef __USE_MPI__
	   MPI_Allreduce(&write_tim, &new_write_tim, 1, MPI_DOUBLE, MPI_MAX,
			MPI_COMM_WORLD);
#else
		new_write_tim = write_tim;
#endif
	   min_write_tim = (new_write_tim < min_write_tim) ? 
			new_write_tim : min_write_tim;
	}
	
	if (mynod == 0) {
			printf("nr_procs = %d, nr_iter = %d, blk_sz = %d\n", nprocs,
		opt_iter, opt_block);
	   write_bw = ((int64_t)(opt_block*nprocs))/(min_write_tim*1000000.0);
	   printf("Write bandwidth = %f Mbytes/sec\n", write_bw);

	   read_bw = ((int64_t)(opt_block*nprocs))/(min_read_tim*1000000.0);
	   printf("Read bandwidth = %f Mbytes/sec\n", read_bw);
		if (opt_correct) {
			printf("Correctness test %s.\n", correct ? "passed" : "failed");
		}
	}
die_jar_jar_die:
	free(tmp);
	if (!opt_correct) free(tmp2);
#ifdef __USE_MPI__
	MPI_Finalize();
#endif
	exit(0);
}

int parse_args(int argc, char **argv)
{
	int c;
	
	while ((c = getopt(argc, argv, "s:b:i:f:c")) != EOF) {
		switch (c) {
			case 's': /* stripe */
				opt_stripe = atoi(optarg);
				break;
			case 'b': /* block size */
				opt_block = atoi(optarg);
				break;
			case 'i': /* iterations */
				opt_iter = atoi(optarg);
				break;
			case 'f': /* filename */
				strncpy(opt_file, optarg, 255);
				break;
			case 'c': /* correctness */
				opt_correct = 1;
				break;
			case '?': /* unknown */
			default:
				break;
		}
	}
}

/* Wtime() - returns current time in sec., in a double */
double Wtime()
{
	struct timeval t;
	
	gettimeofday(&t, NULL);
	return((double)t.tv_sec + (double)t.tv_usec / 1000000);
}





